/*
 * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
 *
 * SPDX-License-Identifier: MPL-2.0
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0.  If a copy of the MPL was not distributed with this
 * file, you can obtain one at https://mozilla.org/MPL/2.0/.
 *
 * See the COPYRIGHT file distributed with this work for additional
 * information regarding copyright ownership.
 */

options {
	listen-on port @PORT@ { 10.53.0.4; };
	transfer-source 10.53.0.4;
	pid-file "named.pid";
	recursion no;
};

key xfrkey {
	algorithm @DEFAULT_HMAC@;
	secret "9999abcd8765";
};

key notifykey4 {
	algorithm @DEFAULT_HMAC@;
	secret "4444abcd8765";
};

zone "test" {
	/*
	 * Notify comes from a different IP address than the primary listening
	 * address, and with a different key.
	 */
	allow-notify { key notifykey4; };
	type secondary;
	primaries { 10.53.0.1 port @PORT@ key xfrkey; };
};

key rndc_key {
	secret "1234abcd8765";
	algorithm @DEFAULT_HMAC@;
};

controls {
	inet 10.53.0.4 port @CONTROLPORT@ allow { any; } keys { rndc_key; };
};
